from Products.eCards.config import PROJECTNAME, ALLTYPES, ALLSKINS, SendECard
from Products.CMFCore.utils import getToolByName

from StringIO import StringIO

def install(self):
    out = StringIO()
    
    # We install our product by running a GS profile.  We use the old-style Install.py module 
    # so that our product works w/ the Quick Installer in Plone 2.5.x
    print >> out, "Installing eCards"
    setup_tool = getToolByName(self, 'portal_setup')
    old_context = setup_tool.getImportContextID()
    setup_tool.setImportContext('profile-Products.eCards:default')
    setup_tool.runAllImportSteps()
    setup_tool.setImportContext(old_context)
    print >> out, "Installed types and added to portal_factory via portal_setup"
    
    propsTool = getToolByName(self, 'portal_properties')
    siteProperties = getattr(propsTool, 'site_properties')
    navtreeProperties = getattr(propsTool, 'navtree_properties')
    
    # Add the eCard types to types excluded from navigation
    # This is not desirable to do with GS because we don't want to maintain a list of 
    # the Portal's metaTypesNotToList and we don't want to overwrite existing settings
    typesNotListed = list(navtreeProperties.getProperty('metaTypesNotToList'))
    for f in ALLTYPES:
        if f not in typesNotListed:
            typesNotListed.append(f)
    navtreeProperties.manage_changeProperties(metaTypesNotToList = typesNotListed)
    print >> out, "Added eCard types to metaTypesNotToList"
    
    # Add the eCard to types using view action in listing
    # This is not desirable to do with GS because we don't want to maintain a list of 
    # the Portal's types list and we don't want to overwrite existing settings
    typesUsingViewAction = list(siteProperties.getProperty('typesUseViewActionInListings'))
    for f in ('eCard',):
        if f not in typesUsingViewAction:
            typesUsingViewAction.append(f)
    siteProperties.manage_changeProperties(typesUseViewActionInListings = typesUsingViewAction)
    print >> out, "Added eCard to typesUseViewActionInListings"
    
    # get the portal and tweak some permissions
    u_tool = getToolByName(self, 'portal_url')
    portal = u_tool.getPortalObject()
    portal.manage_permission(SendECard, ('Manager', 'Anonymous'), acquire=1)
    print >> out, "Configuring Send eCard permission appropriately"
    
    print >> out, "Successfully installed %s." % PROJECTNAME
    return out.getvalue()

def uninstall(self):
    out = StringIO()
    
    portal_factory = getToolByName(self,'portal_factory')
    propsTool = getToolByName(self, 'portal_properties')
    siteProperties = getattr(propsTool, 'site_properties')
    navtreeProperties = getattr(propsTool, 'navtree_properties')
    types_tool = getToolByName(self, 'portal_types')

    # remove eCards as a factory types
    factory_types = portal_factory.getFactoryTypes().keys()
    for t in ALLTYPES:
        if t in factory_types:
            factory_types.remove(t)
    portal_factory.manage_setPortalFactoryTypes(listOfTypeIds=factory_types)
    print >> out, "Removed eCards types from portal_factory tool"

    # remove our types from the portal's list of types excluded from navigation
    typesNotListed = list(navtreeProperties.getProperty('metaTypesNotToList'))
    for f in ALLTYPES:
        if f in typesNotListed:
            typesNotListed.remove(f)
    navtreeProperties.manage_changeProperties(metaTypesNotToList = typesNotListed)
    print >> out, "Removed eCards types from metaTypesNotToList"

    # remove our types from the portal's list of types using view in action listings
    typesUseViewAction = list(siteProperties.getProperty('typesUseViewActionInListings'))
    for f in ('eCard',):
        if f in typesUseViewAction:
            typesUseViewAction.remove(f)
    siteProperties.manage_changeProperties(typesUseViewActionInListings = typesUseViewAction)
    print >> out, "Removed eCard from typesUseViewActionInListings"

    # Remove skin directory from skin selections
    skinstool = getToolByName(self, 'portal_skins')
    for skinName in skinstool.getSkinSelections():
        path = skinstool.getSkinPath(skinName)
        path = [i.strip() for i in  path.split(',')]
        for specific_layer in ALLSKINS:
            if specific_layer in path:
                path.remove(specific_layer)
        path = ','.join(path)
        skinstool.addSkinSelection(skinName, path)

    print >> out, "Removed skin layers from all skin selections"
    
    # clean-up mapped workflow chains for GS configured types
    workflow_tool = getToolByName(self, 'portal_workflow')
    del workflow_tool._chains_by_type['eCardCollection']
    print >> out, "Removed the associated eCardCollection type chains"

    print >> out, "\nSuccessfully uninstalled %s." % PROJECTNAME
    return out.getvalue()
    